{% load static %}
{% if theme_css %}
<style id="spellbook-theme">
{{ theme_css }}
</style>
{% endif %}
<link rel="stylesheet" href="{% static 'django_spellbook/root.css' %}">
<link rel="stylesheet" href="{% static 'django_spellbook/utilities.css' %}">
<link rel="stylesheet" href="{% static 'django_spellbook/styles.css' %}">
